<?php
namespace Drupal\registration_extra\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Drupal\field\Entity\FieldConfig;
use Drupal\registration\Entity\Registration;
use Drupal\registration\Entity\RegistrationType;
use \Symfony\Component\HttpFoundation\StreamedResponse;

class DownloadRegistrationsForm extends \Drupal\registration\Form\RegistrationFormBase {

  public function getFormId() {
    return 'registration_extra_download';
  }

  public function buildForm(array $form, FormStateInterface $form_state, $node = NULL) 
  { 
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download'),
    ];

    return $form;
  }


  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $database = \Drupal::database();
    $entityTypeManager = \Drupal::service('entity_type.manager');
    $host_entity = $this->getHostEntity($form_state);
    //$registration = $this->getEntity();

    $registrationType = $host_entity->getRegistrationType();

    //echo $registration->Id().' '.$host_entity->Id(); die();
    $results = [];
    $query = $database->select('registration', 'r');
    $query->fields('r', [
      'registration_id',
    ]);
    $query->condition('r.entity_type_id', $host_entity->getEntityTypeId());
    $query->condition('r.entity_id', $host_entity->id());
    
    $results = $query->execute();

    //$registrationIds = $results->fetchAllAssoc('registration_id');
    $registrationIds = $results->fetchAllKeyed(0,0);
//print_r($registrationIds); die();

    
    $registrations = Registration::loadMultiple(array_keys($registrationIds));


    //echo 'registration_type = '. $registrationType->bundle();  $bundle_field_name = $registrationType->getEntityType()->Id();
    //print_r($registrationType->toArray()); 
    $fields = [];

    $registrationTypeId = $registrationType->Id();
    //echo '+++'.$registrationType->Id().'++';

    $configFactory = \Drupal::configFactory();
    $fieldConfigNames = $configFactory->listAll('field.field.registration.'.$registrationTypeId);
    $fieldsConfig = $configFactory->loadMultiple($fieldConfigNames);
    foreach ($fieldsConfig as $key => $fieldConfig) {
      //echo '**'.$fieldConfig->getName().' : '.$fieldConfig->get('field_name').'<br>';
      $fields[$fieldConfig->get('field_name')] = $fieldConfig->get('field_name');
    }


    $registration_storage = \Drupal::entityTypeManager()->getStorage('registration_type');

    $response = new StreamedResponse(function () use ($registrations, $fields, $registration_storage) {
      $handle = fopen('php://output', 'a+');
      $BOM = "\xEF\xBB\xBF"; // UTF-8 BOM
      fwrite($handle, $BOM); // NEW LINE
  
      $columns = [
        'registration_id', 'type', 'entity_id', 'entity_label', 'anon_mail', 'user_uid', 'count', 'mail', 'author_uid', 'state', 'created', 'changed', 'completed'
      ];
      $columns += $fields;
      
      fputcsv($handle, $columns, ';', '"');
  
      foreach ($registrations as $key => $registration) 
      {
        //$registration = $registration_storage->load($record->registration_id);
        $author = $registration->getAuthor();
        $user = $registration->getUser();
        $created = $registration->getCreatedTime();
        $changed = $registration->get('changed')->value;
        //$this->dateFormatter->format($registration->getCreatedTime(), 'short')
        $row = [
          'registration_id' => $registration->Id(),
          'type' => $registration->getType()->Id(),
          'entity_id' => $registration->getHostEntity()->Id(),
          'entity_Label' => $registration->getHostEntity()->Label(),
          'anon_mail' => $registration->getAnonymousEmail(),
          'user_uid' => $user ? $author->Id() : null,
          'count' => $registration-> getSpacesReserved(),
          'mail' => $registration->getEmail(),
          'author_uid' => $author ? $author->Id() : null,
          'state' => $registration->getEmail(),
          'created' => date('d/m/Y', $created),
          'changed' => date('d/m/Y', $changed),
          'completed' => $registration->getEmail(),
        ];
        foreach ($fields as $key => $value) {
          $row[$key] = '';
          if( $registration->hasField($key) ) {
            $fieldData = $registration->get($key);

            if($fieldData->entity)
              $row[$key] = $fieldData->entity->Id().' : '.$fieldData->entity->Label();
            else
              $row[$key] = $fieldData->value;
          }
        }

        if($key == 0) {
          fputcsv($handle, $header, ';', '"');
        }
        fputcsv($handle, $row, ';', '"');  
      }
      
      fclose($handle);
    });

    $requestTime =  \Drupal::time()->getCurrentTime();
    $title = 'registration-'.$host_entity->Id(). '_' . $requestTime;
    $fileName = $title . '.csv';

    $response->headers->set('Content-Type', 'application/force-download');
    $response->headers->set('Content-Disposition', 'attachment; filename="' . $fileName . '"');
    
    $form_state->setResponse($response);
  }

private function test() {

echo '<hr>'; 


$config = \Drupal::config('registration.type.formulaire_classique');
echo $config->get('label');
//print_r(array_keys($config));
$configs = \Drupal::config('field.field.registration.formulaire_classique.*');
foreach ($configs as $key => $value) {
  echo '---' . print_r($value->Label());
}
die();

//$info = FieldConfig::loadByName('registration_type', $registrationType->bundle());
//print_r( array_keys($info)  );

$fields = [];
//$fields = RegistrationType::baseFieldDefinitions(RegistrationType::class);


$configFactory = \Drupal::configFactory();
$hh = $configFactory->listAll('field.field.registration.formulaire_classique');
foreach ($hh as $key => $value) {
  echo $value.'<br>';
}


die();
echo '+++'.$registrationType->getConfigDependencyKey().'+++';

$bundle_field_name = $registrationType->getEntityType()->Id();
if (!empty($bundle_field_name)) {
  echo '///'.$bundle_field_name.'//';
  /*$bundle_config = $entity->get($bundle_field_name)->entity;
  if ($bundle_config instanceof \Drupal\mymodule\Entity\WebsiteTypeInterface) {
    $url = $bundle_config->getServiceUrl();
  }*/
}

/*echo $registrationType->Id();
$source_configuration = $registrationType->getSource();
$allowed_source_fields[] = $source_configuration['source_field'];
*/

//$fields += $entity_field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions('registration_type', 'formulaire_classique');
/*
$field_list = [];
foreach ($fields as $field_name => $field_definition) {
  $field_list[$field_name] = [
    'label' => $field_definition->getLabel(),
    'description' => $field_definition->getDescription(),
    'type' => $field_definition->getType(),
  ];
}
print_r($field_list);*/


/*
https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Entity%21EntityTypeManager.php/class/EntityTypeManager/10
https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Config%21Entity%21ConfigEntityBundleBase.php/class/ConfigEntityBundleBase/10
https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Config%21Entity%21ConfigEntityBase.php/class/ConfigEntityBase/10
*/

$registration_type = $entityTypeManager->getStorage('registration_type')->load('formulaire_classique');
echo '**'.$registration_type->getConfigTarget().'**';
$plugin_collections = $registration_type->getDefinitions();

foreach ($fields as $field) {
echo 'ici';
}

/*
$entity_type = 'registration_type';
$container = \Drupal::getContainer();
// @var \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager 
$entity_type_manager = $container->get('entity_type.manager');
$entity_type_definition = $entity_type_manager->getDefinition($entity_type);
$entity_type_class = $entity_type_definition->getClass();
$fields = call_user_func([$entity_type_class, 'baseFieldDefinitions'], $entity_type_definition);
if (!empty($fields['changed'])) {
  // Do something.
}
*/

//$fields = \Drupal\registration\Entity\RegistrationType::baseFieldDefinitions(\Drupal::entityTypeManager()->getDefinition('foo'));


//$registration_storage = \Drupal::entityTypeManager()->getStorage('registration_type');
$fieldDefinitions = \Drupal::service('entity_field.manager')->getFieldDefinitions('registration_type', 'formulaire_classique');

die('end');

    foreach ($fieldDefinitions as $name => $field) {
      //$myFields[$name] = $field->getString();
      echo $name.' ';
    }
    print_r($myFields); die();
}

}