<?php

namespace Drupal\registration_extra;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Drupal\mysql\Driver\Database\mysql\Connection;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

use Drupal\Core\Render\Markup;

class ToolService 
{
	public function __construct(AccountProxyInterface $current_user, SessionInterface $session, Connection $database_connection) {
    $this->currentUser = $current_user;
    $this->session = $session;
    $this->database = $database_connection;
  }
  

  public function buildDisplay($registration)
  {
    $entityTypeManager = \Drupal::entityTypeManager();
    $configFactory = \Drupal::configFactory();

    $registrationType = $registration->getType();
    $registrationTypeId = $registrationType->Id();

    $hostEntity = $registration->getHostEntity();
    $langcode = $registration->getLangcode();
    if(!$langcode)
      $langcode = \Drupal::languageManager()->getDefaultLanguage()->getId();
    
    $registrationSettings = $entityTypeManager->getStorage('registration_settings')->loadSettingsForHostEntity($hostEntity, $langcode);

    $fields = [];
    $fieldConfigNames = $configFactory->listAll('field.field.registration.'.$registrationTypeId);
    $fieldsConfig = $configFactory->loadMultiple($fieldConfigNames);
    foreach ($fieldsConfig as $key => $fieldConfig) {
      //echo '**'.$fieldConfig->get('label').' : '.$fieldConfig->get('field_name').'<br>';
      $fields[$fieldConfig->get('field_name')] = $fieldConfig->get('label');
    }


    $author = $registration->getAuthor();
    $user = $registration->getUser();
    $created = $registration->getCreatedTime();
    $changed = $registration->get('changed')->value;
    //$this->dateFormatter->format($registration->getCreatedTime(), 'short')
    $html = '';

    $html .= '<div>Id: '.$registration->Id().' </div>';
    $html .= '<div>Contenu: '.$registration->getHostEntity()->Label() . ' ('. $registration->getHostEntity()->Id().') </div>';
    $html .= '<div>Email: '.($user ? $registration->getEmail() : $registration->getAnonymousEmail()).' </div>';
    if($user)
      $html .= '<div>Utilisateur: '.$user->Id().' -  '.$user->getDisplayname().' </div>';
    $html .= '<div>Créé le: '.date('d/m/Y', $created).' </div>';

    foreach ($fields as $key => $value) {
      if( $registration->hasField($key)){
        $fieldData = $registration->get($key);
        if($fieldData->entity)
        $html .= '<div>'.$value.': '.$fieldData->entity->Label().' ('.$fieldData->entity->Id().') </div>';
        else
        $html .= '<div>'.$value.':'.$fieldData->value.' </div>';
      }
    }


    return Markup::create($html);

  }
  
}